#ifndef OGGM_H
#define OGGM_H

#define PRAGMA_BIND "oggMedia.dll";

#include <atypes.h>
#include <d3d9.h>



/**\mainpage
 * 
 * \section intro Introduction
 *
 * This is the documentation for <tt>oggMedia</tt> lite-c API.
 * The best fucking plugin for playing ogg theora/vorbis movies inside GameStudio projects.
 * Ogg <a href="http://www.theora.org/">Theora</a> is a free,
 * patent-unencumbered video codec.
 * Ogg <a href="http:://www.vorbis.com/">Vorbis</a> is a completely open, patent-free, professional audio codec.
 *
 * Using oggMedia you don't need to install any strange (most of times not working) codecs on your user computers.
 * It will just work out of the box.
 *
 * \subsection Organization
 *
 * The functions documented here are actually subdivided into four 
 * separate modules:
 * - \ref oggM describes the function which must be called to get oggMedia running
 * - \ref oggM_contr describes the controller interface,
 * - \ref oggM_vorbis the functions used for ogg vorbis
 * - and \ref oggM_theora for ogg theora
 * - \ref oggM_movie contains functions for simplifying the usage of oggMedia
 *
 */

/*! \page tutPage Tutorials
 * Some simple tutorials for using oggMedia.
 * - \subpage setup 
 * - \subpage playmovie 
 * - \subpage playlocalize
*/

/*! \page setup oggMedia setup
 * First you have to copy the following files to your project dir:
 * 	- oggMedia.h
 * 	- ycbcr2rgb.fxo
 * 	- oggMedia.dll
 * 
 * In your main script include oggMedia.h.
 * In your main function call oggM_setup(pd3ddev) after initialisation of the video device (after a wait(1) after video_set/window calls).
 * Call oggM_update() every frame ( while(1) { oggM_update(); wait(1); } )
 *\code
 	function main()
	{
		fps_max = 50;
		wait(2);
		video_window(vector(0, 100, 0), vector(1280, 720,0), 1, "test");
		video_set(1280, 720, 0, 2);
		wait(3);
		// init oggMedia
		oggM_init(pd3ddev);	
		wait(1);
		while(1)
		{
			oggM_update();
			wait(1);	
		}
	}
 \endcode
 * 
 * Now you can proceed to the \ref playmovie "playing a movie section".  
*/


/*! \page playmovie Playing a movie
 * The most simple way to play a movie is using the oggM_movie functions:
 *\code
void blabla()
{
	// create a panel for displaying the movie
	BMAP* movie_bmap = bmap_create(640,480,24);
	Panel* movie_pan = pan_create(0, 1);
	movie_pan->bmap = movie_bmap;
	movie_pan->size_x = 640;
	movie_pan->size_y = 480;
	set(movie_pan, show);
	oggM_moviePlayAndForget("logo.ogg", movie_bmap);
}
\endcode 
 * Now you can proceed to the \ref playlocalize "playing a movie with different audio section".  
*/

/*! \page playlocalize Playing a movie with different audio streams
 * oggMedia allows you to bind a different audio file (you can even specify more than one audio stream) to a video stream.
 * This can be used to play localized audio streams which can save alot of space.
 *\code
void playCutscene(int language)
{
	// create a panel for displaying the movie
	BMAP* movie_bmap = bmap_create(640,480,24);
	Panel* movie_pan = pan_create(0, 1);
	movie_pan->bmap = movie_bmap;
	movie_pan->size_x = 640;
	movie_pan->size_y = 480;
	set(movie_pan, show);
	STRING* audio_file;
	if (language == 0)	str_cpy(audio_file, "english.oga");
	if (language == 1)	str_cpy(audio_file, "german.oga");
	if (language == 2)	str_cpy(audio_file, "french.oga");
	long movie = oggM_movieOpen("intro1.ogv", movie_bmap, 0, audio_file->chars);
	if (movie == 0)	 return;	//there was an error while opening
	oggM_moviePlay(movie);
	while (oggM_moivePlaying(movie))	wait(1);
	oggM_movieFree(movie);
}
\endcode 
*/

/*! \page legal License notes
* oggMedia is beeing developed by Christopher <ChrisB> Bl&auml;sius and uses code copyrighted by Xiph.org Foundation.
* 
* oggMedia is currently in a beta phase, which means it shouldn't be used for any commercial project. 
* Each beta version will stop working a month after its release.
* Final releases may contain a different license, but for now its free for everyone.
*
* If you want to use oggMedia in your product you have to include the following copyright notice somewhere in your product,
* for example somewhere in your credit screen, in your readme files or even as a startup notice.
*
* \section legal
Copyright (C) 2002-2009 Xiph.org Foundation

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither the name of the Xiph.org Foundation nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
\endsection
*/


typedef struct
{
	double time;		///< current position in seconds
	double length;		///< total length in secons
	int state;			///< state of the controller (0 stopped, 1 playing, 2 paused), not specified for streams
	int preload;		///< 1 if the stream is preloaded, not specified for controllers
} MediaStatus;

typedef struct
{
	int width;							///< width in pixels of the theora video
	int height;							///< height in pixels of the theora video
	int fps;								///< frames per seconds
	float aspectRatio;				///< aspect ratio of the video.
											///< Some videos uses anamorphic widescreen which results in not quadratic pixels. This must be considered for playback.
											///< For example 16:9 dvd are encoded with 720x576 (which is 5:4), aspectRatio would be ~1.4
	float length;						///< total length of the theora stream in seconds
	LPDIRECT3DTEXTURE9 texture;	///< pointer to the texture containing the video playback
} TheoraInfo;


/**
 * \defgroup oggM Setup functions
 * Functions that are needed to get oggMedia running.
 */
/*@{*/

/**
* @brief Initialiaze oggMedia
*
* Call this after startup, the video device must be already open.
*
* @param device pointer to the Direct3D device, pd3ddev
* @return 0
*/
long oggM_init(LPDIRECT3DDEVICE9 device);

/**
* @brief Update oggMedia
*
* Call this every frame.
*
* @return 0
*/
long oggM_update();

/*@}*/




/**
 * \defgroup oggM_contr Controller
 *
 */
/*@{*/
/**\name Functions to controll theora and vorbis streams.

* The controller links multiple vorbis and theora streams together and controlls the playback.
*/
/*@{*/

/**
* @brief Creates controller
*
* Creates a controller
*
* @return handle to new controller or 0 if something failed
*/
long oggM_contrCreate();

/**
* @brief Adds a new audio stream
*
* Add a new vorbis audio stream to the controller.
*
* @param hndlContr handle to the controller
* @param hndlVorb handle to the vorbis stream
* @offset offset of the vorbis stream (can be 0, but not negative)
* @return 1, or 0 if something failed
*/
long oggM_contrAddVorbis(long hndlContr, long hndlVorb, long offset);

/**
* @brief Adds a new video stream
*
* Add a new theora video stream to the controller.
*
* @param hndlContr handle to the controller
* @param hndlTheora handle to the theora stream
* @offset offset of the theora stream (can be 0, but not negative)
* @return 1, or 0 if something failed
*/
long oggM_contrAddTheora(long hndlContr, long hndlTheora, long offset);

/**
* @brief Play
*
* Starts playing the streams linked to the controller.
* A paused controller will start playing from the paused position
*
* @param hndl handle to the controller
* @return 1, or 0 if something failed
*/
long oggM_contrPlay(long hndl);

/**
* @brief Pause
*
* Pauses the streams
*
* @param hndl handle to the controller
* @return 1, or 0 if something failed
*/
long oggM_contrPause(long hndl);


/**
* @brief Stop
*
* Stops the stream. Calling Play again will start the streams from beginning.
*
* @param hndl handle to the controller
* @return 1, or 0 if something failed
*/
long oggM_contrStop(long hndl);

/**
* @brief Free
*
* Frees all streams linked to this controller and itself. Handles will get invalid.
*
* @param hndl handle to the controller
* @return 1, or 0 if something failed
*/
long oggM_contrFree(long hndl);

/**
* @brief SetLength
*
* Sets the total length of the controller. Should be set to the length of one of its streams.
* Playback will stop when receiving the position specified by length
*
* @param hndl handle to the controller
* @param length length of streams in seconds
* @return 1, or 0 if something failed
*/
long oggM_contrSetLength(long hndl, double length);

/**
* @brief Get status informations
*
* @param hndl handle to the controller
* @param state (out) pointer to a MediaStatus object that will contain the result
* @return 1, or 0 if something failed
*/
long oggM_contrStatus(long hndl, MediaStatus* state);
/*@}*/
/*@}*/





/**
 * \defgroup oggM_vorbis Vorbis audio*/
/*@{*/
/**\name Functions for vorbis audio
* Vorbis audio streams are encoded in .ogg.
* For playing a single vorbis stream call the following functions:
*	- oggM_contrCreate()
*	- oggM_vorbisCreate()
*	- oggM_contrAddVorbis()
*	- oggM_vorbisStatus()
*	- oggM_contrSetLength()
*	- oggM_contrPlay()
*	- and oggM_contrFree() to close all streams and handles afterwards.
*
*/
/*@{*/

/**
* @brief Creates vorbis stream
*
* Opens an ogg vorbis file and creates a stream.
*
* @param str String containing the filename
* @return handle to new vorbis stream or 0 if something failed
*/
long oggM_vorbisCreate(char* str);

/**
* @brief Get status informations
*
* @param hndl handle to the vorbis stream
* @param state (out) pointer to a MediaStatus object that will contain the result
* @return 1
*/
long oggM_vorbisStatus(long hndl, MediaStatus* state);

/**
* @brief Free
*
* Frees the vorbis stream. The stream will get automatically removed from its controller and the handle will get invalid.
*
* @param hndl handle to the vorbis stream
* @return 1
*/
long oggM_vorbisFree(long hndl);
/*@}*/
/*@}*/

/**
 * \defgroup oggM_theora Theora video
 */
/*@{*/
/**\name Functions for theora video
* For playing a single theora stream call the following functions:
*	- oggM_contrCreate()
*	- oggM_theoraCreate()
*	- oggM_contrAddTheora()
*	- oggM_theoraStatus()
*	- oggM_contrSetLength()
*	- oggM_contrPlay()
*	- and oggM_contrFree() to close all streams and handles afterwards.
*
*/
/*@{*/

/**
* @brief Creates theora stream
*
* Opens an ogg theora file and creates a stream.
*
* @param str String containing the filename
* @return handle to new theora stream or 0 if something failed
*/
long oggM_theoraCreate(char* str);

/**
* @brief Get status informations
*
* @param hndl handle to the theora stream
* @param state (out) pointer to a MediaStatus object that will contain the result
* @return 1
*/
long oggM_theoraStatus(long hndl, MediaStatus* state);

/**
* @brief Get more informations
*
* @param hndl handle to the theora stream
* @param info (out) pointer to a TheoraInfo object that will contain the result
* @return 1
*/
long oggM_theoraInfo(long hndl, TheoraInfo* info);

/**
* Frees the theora stream. The stream will get automatically removed from its controller and the handle will get invalid.
*
* @param hndl handle to the theora stream
* @return 1
*/
long oggM_theoraFree(long hndl);



/**
* @brief Creates a theora stream
*
* Creates and opens a theora stream with the given filename. target_bmp will be used for playback.
*
* @param str String containing the filename
* @param target_bmp pointer to a BMAP that will be used for playback
* @return handle to new theora stream or 0 if something failed
*/
long oggM_theoraOpen(char* file, BMAP* target_bmp)
{
	long theora = oggM_theoraCreate(file);
	if (target_bmp == 0)	return theora;	
	if (theora != 0) {
		TheoraInfo tInfo;
		oggM_theoraInfo(theora, &tInfo);	
		bmap_preload(target_bmp);
		((LPDIRECT3DTEXTURE9)(target_bmp->d3dtex))->Release();
		target_bmp->d3dtex = tInfo.texture;
		((LPDIRECT3DTEXTURE9)(target_bmp->d3dtex))->AddRef();
		target_bmp->width = tInfo.width;
		target_bmp->height = tInfo.height;
		target_bmp->finalwidth = tInfo.width;
		target_bmp->finalheight = tInfo.height;
	}
	return theora;
}
/*@}*/
/*@}*/
//


#define PRAGMA_API oggM_init;oggMedia!oggM_init
#define PRAGMA_API oggM_update;oggMedia!oggM_update

#define PRAGMA_API oggM_contrCreate;oggMedia!oggM_contrCreate
#define PRAGMA_API oggM_contrAddVorbis;oggMedia!oggM_contrAddVorbis
#define PRAGMA_API oggM_contrAddTheora;oggMedia!oggM_contrAddTheora
#define PRAGMA_API oggM_contrPlay;oggMedia!oggM_contrPlay
#define PRAGMA_API oggM_contrPause;oggMedia!oggM_contrPause
#define PRAGMA_API oggM_contrStop;oggMedia!oggM_contrStop
#define PRAGMA_API oggM_contrFree;oggMedia!oggM_contrFree
#define PRAGMA_API oggM_contrSetLength;oggMedia!oggM_contrSetLength
#define PRAGMA_API oggM_contrStatus;oggMedia!oggM_contrStatus

#define PRAGMA_API oggM_vorbisCreate;oggMedia!oggM_vorbisCreate
#define PRAGMA_API oggM_vorbisStatus;oggMedia!oggM_vorbisStatus
#define PRAGMA_API oggM_vorbisFree;oggMedia!oggM_vorbisFree

#define PRAGMA_API oggM_theoraCreate;oggMedia!oggM_theoraCreate
#define PRAGMA_API oggM_theoraStatus;oggMedia!oggM_theoraStatus
#define PRAGMA_API oggM_theoraInfo;oggMedia!oggM_theoraInfo
#define PRAGMA_API oggM_theoraFree;oggMedia!oggM_theoraFree

/**
 * \defgroup oggM_movie Movie
 *
 */
/*@{*/
/**\name Functions to simplifyng calls.
* General purpose functions, which simplifies some calls.
* Most of the times you only need these functions, for example to play a simple video file with audio.
*
* To play a movie with embedded sound you just have to call oggM_moviePlayAndForget(), it will start playing immediately and frees the stream structures when finished.
*
* The handles used by the oggM_movie... functions are handles to controllers, so you can also use oggM_contr... functions with them (see \ref oggM_contr)
*
*/
/*@{*/


/**
* @brief Creates a new movie stream.
*
* Creates and opens a new movie stream. A different file for audio playback can be specified, including an offset to the video stream.
*
* @param file String containing the filename
* @param target_bmp pointer to a BMAP that will be used for playback
* @param tInfo (out) pointer to a TheoraInfo object which will receive infos about the theora stream. Can be 0
* @param audioFile string containing the filename to be used for audio. If the pointer is 0 the char* file will be used for audio playback.
* @param offset offset used for audio playback.
* @return handle to new movie or 0 if something failed
*/
long oggM_movieOpen(char* file, BMAP* target_bmp, TheoraInfo* tInfo, char* audioFile, double offset)
{
	MediaStatus state;
	long contr = oggM_contrCreate();
	if (contr == 0)	return 0;
	long theora = oggM_theoraOpen(file, target_bmp);
	if (theora != 0) {
		if (tInfo != 0) oggM_theoraInfo(theora, tInfo);	
		oggM_contrAddTheora(contr, theora, 0);
		oggM_theoraStatus(theora, &state);
		oggM_contrSetLength(contr, state.length);
	}
	long vorbis = 0;
	if (audioFile == 0)
		vorbis = oggM_vorbisCreate(file);
	else
		vorbis = oggM_vorbisCreate(audioFile);		
	if (vorbis != 0)
		oggM_contrAddVorbis(contr, vorbis, offset);	
	return contr;
}

/**
* @brief Creates a new movie stream.
*
* Overloaded version of oggM_movieOpen()
*/
long oggM_movieOpen(char* file, BMAP* target_bmp, TheoraInfo* tInfo, char* audioFile)
{
	oggM_movieOpen(file, target_bmp, tInfo, audioFile, 0);
}

/**
* @brief Creates a new movie stream.
*
* Overloaded version of oggM_movieOpen()
*/
long oggM_movieOpen(char* file, BMAP* target_bmp, TheoraInfo* tInfo)
{
	oggM_movieOpen(file, target_bmp, tInfo, 0, 0);
}

/**
* @brief Creates a new movie stream.
*
* Overloaded version of oggM_movieOpen()
*/
long oggM_movieOpen(char* file, BMAP* target_bmp)
{
	oggM_movieOpen(file, target_bmp, 0, 0, 0);
}

/**
* @brief Play
*
* Starts playing the movie.
* A paused movie will start playing from the paused position
*
* @param hndl handle to the movie stream.
* @return 1, 0 if something failed.
*/
long oggM_moviePlay(long hndl)
{
	return oggM_contrPlay(hndl);	
}

/**
* @brief Pauses
*
* Pauses the movie.
*
* @param hndl handle to the movie stream.
* @return 1, 0 if something failed.
*/
long oggM_moviePause(long hndl)
{
	return oggM_contrPause(hndl);	
}

/**
* @brief Stop
*
* Stops playing the movie.
* The play position will be set to the beginning of the movie.
*
* @param hndl handle to the movie stream.
* @return 1, 0 if something failed.
*/
long oggM_movieStop(long hndl)
{
	return oggM_contrStop(hndl);	
}


/**
* @brief Returns the total length
*
* @param hndl handle to the movie stream.
* @return total length of the movie in seconds, -1 if something failed.
*/
double oggM_movieLength(long hndl)
{
	MediaStatus state;		
	if (oggM_contrStatus(hndl, &state))
	{
		return state.length;	
	}
	return -1;
}

/**
* @brief Returns the current position
*
* @param hndl handle to the movie stream.
* @return current position of the movie in seconds, -1 if something failed.
*/
double oggM_movieTime(long hndl)
{
	MediaStatus state;		
	if (oggM_contrStatus(hndl, &state))
	{
		return state.time;	
	}
	return -1;
}

/**
* @brief Test if movie is playing
*
* @param hndl handle to the movie stream.
* @return 0 if movie is stopped, 1 for playing, 2 for paused, -1 if something failed.
*/
int oggM_moviePlaying(long hndl)
{
	MediaStatus state;		
	if (oggM_contrStatus(hndl, &state))
	{
		return state.state;	
	}
	return -1;
}

/**
* @brief Free
*
* Frees everything used by the movie. Handles will get invalid.
*
* @return 1
*/
int oggM_movieFree(long hndl)
{
	return oggM_contrFree(hndl);
	return -1;
}

/**
* @brief Creates a new movie stream.
*
* Creates and opens a new movie stream and starts playing it immediately.
* The movie will get freed after playing has finished.
*
* @param file String containing the filename
*/
void oggM_moviePlayAndForget(char* file,  BMAP* target_bmp)
{
	long movie = oggM_movieOpen(file, target_bmp);
	if (movie == 0)	return;
	oggM_moviePlay(movie);
	while (oggM_moviePlaying(movie))	wait(1);
	oggM_movieFree(movie);
}

/*@}*/	
/*@}*/
//

#endif